@@ -32,12 +32,21 @@ from .constants import (
     CURRENT_PAGE_POSITION,
     DOOR_TRANSITION_TIMER,
     ENEMIES_DEFEATED,
+    ENEMY_DEAD,
     ENEMY_HEALTH,
     ENEMY_ID,
+    ENEMY_INVISIBLE,
+    ENEMY_NOT_PRESENT,
+    ENEMY_SPEED,
+    ENEMY_VISIBILITY,
+    ENEMY_VISIBLE,
+    ENEMY_X_PAGE,
     ENEMY_X_POS,
+    ENEMY_Y_PAGE,
     ENEMY_Y_POS,
     FLOAT_TIMER,
     GAME_INIT_FRAMES,
+    INVULNERABILITY_TIMER,
     ITEM_HOLDING,
     ITEM_PULLED,
     LEVEL_NAMES,
@@ -76,7 +85,6 @@ from .constants import (
     TOTAL_PAGES_IN_SUB_AREA,
     VEGETABLES_PULLED,
     WORLD_NUMBER,
-    Y_POSITION_WRAPAROUND_THRESHOLD,
     GlobalCoordinate,
 )
 
@@ -102,6 +110,7 @@ class SuperMarioBros2Env(gym.Env):
         reset_on_life_loss: bool = False,
         render_fps: Optional[int] = None,
         frame_method: str = "rgb",
+        env_name: Optional[str] = None,
     ):
         """Initialize the SMB2 environment.
 
@@ -123,6 +132,9 @@ class SuperMarioBros2Env(gym.Env):
         self.reset_on_life_loss = reset_on_life_loss
         self.init_config = init_config
         self.render_fps = render_fps
+        self.env_name = env_name
+        if self.env_name:
+            print(f'Creating {self.env_name} environment...')
 
         # Validate and store frame method
         valid_frame_methods = ["rgb", "grayscale"]
@@ -213,6 +225,7 @@ class SuperMarioBros2Env(gym.Env):
         if self._pygame_initialized or self.render_mode != 'human':
             return
 
+        # Lazy load this, we don't need for non rendered envs
         import pygame
         pygame.init()
 
@@ -388,6 +401,18 @@ class SuperMarioBros2Env(gym.Env):
             return obs
         return None
 
+    def _get_y_position(self, address: int) -> int:
+        """Safely read Y position from RAM and clamp to valid range.
+
+        Args:
+            address: RAM address to read Y position from
+
+        Returns:
+            Y position clamped to 0-239 range
+        """
+        y_pos = self._read_ram_safe(address, default=0)
+        return max(0, min(y_pos, SCREEN_HEIGHT - 1))
+
     def _read_ram_safe(self, address: int, default: int = 0) -> int:
         """Safely read from RAM with fallback.
 
@@ -435,6 +460,7 @@ class SuperMarioBros2Env(gym.Env):
             'starman_timer': self.starman_timer,
             'subspace_timer': self.subspace_timer,
             'stopwatch_timer': self.stopwatch_timer,
+            'invulnerability_timer': self.invulnerability_timer,
             'holding_item': self.holding_item,
             'item_pulled': self.item_pulled,
             'continues': self.continues,
@@ -448,6 +474,17 @@ class SuperMarioBros2Env(gym.Env):
             'level_completed': self.level_completed,
             'door_transition_timer': self.door_transition_timer,
             'enemies_defeated': self.enemies_defeated,
+            'enemy_x_positions': self.enemy_x_positions,
+            'enemy_y_positions': self.enemy_y_positions,
+            'enemy_speeds': self.enemy_speeds,
+            'enemy_visibility_states': self.enemy_visibility_states,
+            'enemy_x_positions_global': self.enemy_x_positions_global,
+            'enemy_y_positions_global': self.enemy_y_positions_global,
+            'enemy_x_pages': self.enemy_x_pages,
+            'enemy_y_pages': self.enemy_y_pages,
+            'enemy_x_positions_relative': self.enemy_x_positions_relative,
+            'enemy_y_positions_relative': self.enemy_y_positions_relative,
+            'enemy_hp': self.enemy_hp,
         }
 
     @property
@@ -506,31 +543,49 @@ class SuperMarioBros2Env(gym.Env):
         x_page = self._read_ram_safe(PLAYER_X_PAGE)
         return x_page
 
+    def _transform_y_coordinate(self, y_page: int, y_pos_raw: int) -> int:
+        """Transform raw Y coordinates to inverted system (y=0 at bottom, increasing upward).
+
+        Args:
+            y_page: Y page value from RAM
+            y_pos: Y position value from RAM
+
+        Returns:
+            Inverted Y coordinate
+        """
+        # Handle wraparound: when goes above top, y_page becomes 255
+        if y_page == 255:
+            y_page = 0
+
+        y_pos_global: int = y_page * SCREEN_HEIGHT + y_pos_raw
+
+        if self.is_vertical_area:
+            max_y_in_level = self.total_pages_in_sub_area * SCREEN_HEIGHT
+        else:
+            max_y_in_level = SCREEN_HEIGHT
+
+        return max_y_in_level - y_pos_global - 1
+
     @property
     def y_position_global(self) -> int:
-        """Get player global Y position."""
+        """Get player global Y position (with y=0 at bottom, increasing upward)."""
         y_page = self._read_ram_safe(PLAYER_Y_PAGE, default=0)
-        y_pos = self._read_ram_safe(PLAYER_Y_POSITION, default=0)
-        global_position = (y_page * PAGE_SIZE) + y_pos
-
-        # NOTE: Handle out-of-bounds positions (wraparound when Mario goes above
-        # screen). Valid Y positions in SMB2 are typically 0-1000, anything > 10000
-        # is wraparound
-        if global_position > Y_POSITION_WRAPAROUND_THRESHOLD:
-            return 0
-
-        return global_position
+        y_pos_raw = self._get_y_position(PLAYER_Y_POSITION)
+        return self._transform_y_coordinate(y_page, y_pos_raw)
 
     @property
     def y_position(self) -> int:
-        """Get player local Y position (on current page)."""
-        y_pos = self._read_ram_safe(PLAYER_Y_POSITION, default=0)
-        return y_pos
+        """Get player local Y position (with y=0 at bottom, increasing upward)."""
+        y_pos = self._get_y_position(PLAYER_Y_POSITION)
+        # Invert the y-coordinate within the screen space
+        return SCREEN_HEIGHT - 1 - y_pos
 
     @property
     def y_page(self) -> int:
         """Get the Y page of the player position."""
         y_page = self._read_ram_safe(PLAYER_Y_PAGE)
+        if y_page == 255:  # Screen wrap around
+            return 0
         return y_page
 
     @property
@@ -572,13 +627,13 @@ class SuperMarioBros2Env(gym.Env):
     def total_pages_in_sub_area(self) -> int:
         """Get total number of pages in the current sub-area."""
         total_pages = self._read_ram_safe(TOTAL_PAGES_IN_SUB_AREA, default=0)
-        return total_pages
+        return total_pages + 1  # zero indexed
 
     @property
     def is_vertical_area(self) -> bool:
         """Check if current area has vertical scrolling."""
         direction = self._read_ram_safe(SCROLL_DIRECTION, default=0)
-        return direction == 1
+        return not bool(direction)
 
     @property
     def global_coordinate_system(self) -> GlobalCoordinate:
@@ -633,7 +688,6 @@ class SuperMarioBros2Env(gym.Env):
     def cherries(self) -> int:
         """Get cherries collected."""
         cherries = self._read_ram_safe(CHERRIES, default=0)
-        # Cherries should be 0-19 (max per level)
         if 0 <= cherries <= MAX_CHERRIES:
             return cherries
         return 0
@@ -642,7 +696,6 @@ class SuperMarioBros2Env(gym.Env):
     def coins(self) -> int:
         """Get coins collected in Subspace."""
         coins = self._read_ram_safe(SUBSPACE_COINS, default=0)
-        # Coins should be reasonable
         if 0 <= coins <= MAX_COINS:
             return coins
         return 0
@@ -662,6 +715,11 @@ class SuperMarioBros2Env(gym.Env):
         """Get stopwatch timer."""
         return self._read_ram_safe(STOPWATCH_TIMER, default=0)
 
+    @property
+    def invulnerability_timer(self) -> int:
+        """Get invulnerability timer (time left until character becomes vulnerable)."""
+        return self._read_ram_safe(INVULNERABILITY_TIMER, default=0)
+
     @property
     def enemies_defeated(self) -> int:
         """Get count of enemies defeated (for heart spawning)."""
@@ -687,8 +745,9 @@ class SuperMarioBros2Env(gym.Env):
 
     @property
     def player_speed(self) -> int:
-        """Get player horizontal speed."""
-        return self._read_ram_safe(PLAYER_SPEED, default=0)
+        """Get player horizontal speed (signed: positive=right, negative=left)."""
+        speed = self._read_ram_safe(PLAYER_SPEED, default=0)
+        return speed if speed < 128 else speed - 256
 
     @property
     def on_vine(self) -> bool:
@@ -744,7 +803,7 @@ class SuperMarioBros2Env(gym.Env):
         3 - end level, go to bonus game (level completed)
         4 - warp
         """
-        # This will almost always return 0 due to sub-frame clearing
+        # This will almost always return 0 due to sub-frame clearing? TODO: Delete
         return self._read_ram_safe(LEVEL_TRANSITION, default=0)
 
     @property
@@ -765,6 +824,214 @@ class SuperMarioBros2Env(gym.Env):
                 return True
         return False
 
+    @property
+    def enemy_visibility_states(self) -> list[int]:
+        """Get visibility states of enemies 1-5.
+
+        Returns:
+            List of 5 enemy visibility states (index 0 = enemy 5, index 4 = enemy 1)
+            0 = Invisible, 1 = Visible, 2 = Dead
+        """
+        return [self._read_ram_safe(addr, default=0) for addr in ENEMY_VISIBILITY]
+
+    @property
+    def enemy_x_positions(self) -> list[int]:
+        """Get X positions of enemies 1-5 on the current page.
+
+        Returns:
+            List of 5 enemy X positions (index 0 = enemy 5, index 4 = enemy 1)
+            Returns ENEMY_NOT_PRESENT for invisible or dead enemies
+        """
+        positions = []
+        visibility_states = self.enemy_visibility_states
+        for i, addr in enumerate(ENEMY_X_POS):
+            if visibility_states[i] in [ENEMY_INVISIBLE, ENEMY_DEAD]:
+                positions.append(ENEMY_NOT_PRESENT)
+            else:
+                positions.append(self._read_ram_safe(addr, default=0))
+        return positions
+
+    @property
+    def enemy_y_positions(self) -> list[int]:
+        """Get Y positions of enemies 1-5 on the current page (with y=0 at bottom).
+
+        Returns:
+            List of 5 enemy Y positions (index 0 = enemy 5, index 4 = enemy 1)
+            Returns ENEMY_NOT_PRESENT for invisible or dead enemies
+        """
+        positions = []
+        visibility_states = self.enemy_visibility_states
+        for i, addr in enumerate(ENEMY_Y_POS):
+            if visibility_states[i] in [ENEMY_INVISIBLE, ENEMY_DEAD]:
+                positions.append(ENEMY_NOT_PRESENT)
+            else:
+                y_pos = self._get_y_position(addr)
+                # Invert the y-coordinate within the screen space
+                positions.append(SCREEN_HEIGHT - 1 - y_pos)
+        return positions
+
+    @property
+    def enemy_speeds(self) -> list[int]:
+        """Get horizontal speeds of enemies 1-5 (signed: positive=right, negative=left).
+
+        Returns:
+            List of 5 enemy speeds (index 0 = enemy 5, index 4 = enemy 1)
+            Returns ENEMY_NOT_PRESENT for invisible or dead enemies
+        """
+        speeds = []
+        visibility_states = self.enemy_visibility_states
+        for i, addr in enumerate(ENEMY_SPEED):
+            if visibility_states[i] in [ENEMY_INVISIBLE, ENEMY_DEAD]:
+                speeds.append(ENEMY_NOT_PRESENT)
+            else:
+                speed = self._read_ram_safe(addr, default=0)
+                signed_speed = speed if speed < 128 else speed - 256
+                speeds.append(signed_speed)
+        return speeds
+
+    @property
+    def enemy_x_pages(self) -> list[int]:
+        """Get X pages of enemies 1-5.
+
+        Returns:
+            List of 5 enemy X pages (index 0 = enemy 5, index 4 = enemy 1)
+            Returns ENEMY_NOT_PRESENT for invisible or dead enemies
+        """
+        pages = []
+        visibility_states = self.enemy_visibility_states
+        for i, addr in enumerate(ENEMY_X_PAGE):
+            if visibility_states[i] in [ENEMY_INVISIBLE, ENEMY_DEAD]:
+                pages.append(ENEMY_NOT_PRESENT)
+            else:
+                pages.append(self._read_ram_safe(addr, default=0))
+        return pages
+
+    @property
+    def enemy_y_pages(self) -> list[int]:
+        """Get Y pages of enemies 1-5.
+
+        Returns:
+            List of 5 enemy Y pages (index 0 = enemy 5, index 4 = enemy 1)
+            Returns ENEMY_NOT_PRESENT for invisible or dead enemies
+        """
+        pages = []
+        visibility_states = self.enemy_visibility_states
+        for i, addr in enumerate(ENEMY_Y_PAGE):
+            if visibility_states[i] in [ENEMY_INVISIBLE, ENEMY_DEAD]:
+                pages.append(ENEMY_NOT_PRESENT)
+            else:
+                pages.append(self._read_ram_safe(addr, default=0))
+        return pages
+
+    @property
+    def enemy_x_positions_global(self) -> list[int]:
+        """Get global X positions of enemies 1-5.
+
+        Returns:
+            List of 5 enemy global X positions (index 0 = enemy 5, index 4 = enemy 1)
+            Returns ENEMY_NOT_PRESENT for invisible or dead enemies
+        """
+        global_positions = []
+        visibility_states = self.enemy_visibility_states
+
+        for i, (x_pos_addr, x_page_addr) in enumerate(zip(ENEMY_X_POS, ENEMY_X_PAGE)):
+            if visibility_states[i] in [ENEMY_INVISIBLE, ENEMY_DEAD]:
+                global_positions.append(ENEMY_NOT_PRESENT)
+            else:
+                x_page = self._read_ram_safe(x_page_addr, default=0)
+                x_pos = self._read_ram_safe(x_pos_addr, default=0)
+                global_x = (x_page * PAGE_SIZE) + x_pos
+                global_positions.append(global_x)
+        return global_positions
+
+    @property
+    def enemy_y_positions_global(self) -> list[int]:
+        """Get global Y positions of enemies 1-5 (with y=0 at bottom, increasing upward).
+
+        Returns:
+            List of 5 enemy global Y positions (index 0 = enemy 5, index 4 = enemy 1)
+            Returns ENEMY_NOT_PRESENT for invisible or dead enemies
+        """
+        global_positions = []
+        visibility_states = self.enemy_visibility_states
+
+        for i, (y_pos_addr, y_page_addr) in enumerate(zip(ENEMY_Y_POS, ENEMY_Y_PAGE)):
+            if visibility_states[i] in [ENEMY_INVISIBLE, ENEMY_DEAD]:
+                global_positions.append(ENEMY_NOT_PRESENT)
+            else:
+                y_page = self._read_ram_safe(y_page_addr, default=0)
+                y_pos_raw = self._get_y_position(y_pos_addr)
+                inverted_y = self._transform_y_coordinate(y_page, y_pos_raw)
+                global_positions.append(inverted_y)
+        return global_positions
+
+    @property
+    def enemy_x_positions_relative(self) -> list[int]:
+        """Get X positions of enemies 1-5 relative to player using global coordinates.
+
+        Returns:
+            List of 5 enemy X positions relative to player (index 0 = enemy 5, index 4 = enemy 1)
+            Returns ENEMY_NOT_PRESENT for invisible or dead enemies
+        """
+        relative_positions = []
+        visibility_states = self.enemy_visibility_states
+        player_x_global = self.x_position_global
+        enemy_x_global = self.enemy_x_positions_global
+
+        for i in range(len(visibility_states)):
+            if visibility_states[i] in [ENEMY_INVISIBLE, ENEMY_DEAD]:
+                relative_positions.append(ENEMY_NOT_PRESENT)
+            else:
+                if enemy_x_global[i] != ENEMY_NOT_PRESENT:
+                    relative_positions.append(player_x_global - enemy_x_global[i])
+                else:
+                    relative_positions.append(ENEMY_NOT_PRESENT)
+        return relative_positions
+
+    @property
+    def enemy_y_positions_relative(self) -> list[int]:
+        """Get Y positions of enemies 1-5 relative to player using global coordinates.
+
+        With inverted Y coordinates (y=0 at bottom, increasing upward):
+        - Positive values = enemy is above player (enemy has higher Y)
+        - Negative values = enemy is below player (enemy has lower Y)
+
+        Returns:
+            List of 5 enemy Y positions relative to player (index 0 = enemy 5, index 4 = enemy 1)
+            Returns ENEMY_NOT_PRESENT for invisible or dead enemies
+        """
+        relative_positions = []
+        visibility_states = self.enemy_visibility_states
+        player_y_global = self.y_position_global
+        enemy_y_global = self.enemy_y_positions_global
+
+        for i in range(len(visibility_states)):
+            if visibility_states[i] in [ENEMY_INVISIBLE, ENEMY_DEAD]:
+                relative_positions.append(ENEMY_NOT_PRESENT)
+            else:
+                if enemy_y_global[i] != ENEMY_NOT_PRESENT:
+                    relative_positions.append(player_y_global - enemy_y_global[i])
+                else:
+                    relative_positions.append(ENEMY_NOT_PRESENT)
+        return relative_positions
+
+    @property
+    def enemy_hp(self) -> list[int]:
+        """Get HP values of enemies 1-5.
+
+        Returns:
+            List of 5 enemy HP values (index 0 = enemy 5, index 4 = enemy 1)
+            Returns ENEMY_NOT_PRESENT for invisible or dead enemies
+        """
+        hp_values = []
+        visibility_states = self.enemy_visibility_states
+        for i, addr in enumerate(ENEMY_HEALTH):
+            if visibility_states[i] in [ENEMY_INVISIBLE, ENEMY_DEAD]:
+                hp_values.append(ENEMY_NOT_PRESENT)
+            else:
+                hp_values.append(self._read_ram_safe(addr, default=0))
+        return hp_values
+
     # ---- Validators ------------------------------------------------
 
     def _validate_and_convert_action(self, action: int) -> np.ndarray:
